@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* $Id$ */
+/* $Id: items.c 745 2008-03-03 07:51:02Z dsallings $ */
 #include "memcached.h"
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -17,6 +17,7 @@
 /* Forward Declarations */
 static void item_link_q(item *it);
 static void item_unlink_q(item *it);
+static uint64_t get_cas_id();
 
 /*
  * We only reposition items in the LRU queue if they haven't been repositioned
@@ -28,9 +29,7 @@ static void item_unlink_q(item *it);
 #define LARGEST_ID 255
 typedef struct {
     unsigned int evicted;
-    rel_time_t evicted_time;
     unsigned int outofmemory;
-    unsigned int tailrepairs;
 } itemstats_t;
 
 static item *heads[LARGEST_ID];
@@ -126,7 +125,6 @@ item *do_item_alloc(char *key, const size_t nkey, const int flags, const rel_tim
             if (search->refcount == 0) {
                 if (search->exptime == 0 || search->exptime > current_time) {
                     itemstats[id].evicted++;
-                    itemstats[id].evicted_time = current_time - search->time;
                     STATS_LOCK();
                     stats.evictions++;
                     STATS_UNLOCK();
@@ -138,26 +136,7 @@ item *do_item_alloc(char *key, const size_t nkey, const int flags, const rel_tim
         it = slabs_alloc(ntotal, id);
         if (it == 0) {
             itemstats[id].outofmemory++;
-            /* Last ditch effort. There is a very rare bug which causes
-             * refcount leaks. We've fixed most of them, but it still happens,
-             * and it may happen in the future.
-             * We can reasonably assume no item can stay locked for more than
-             * three hours, so if we find one in the tail which is that old,
-             * free it anyway.
-             */
-            tries = 50;
-            for (search = tails[id]; tries > 0 && search != NULL; tries--, search=search->prev) {
-                if (search->refcount != 0 && search->time + 10800 < current_time) {
-                    itemstats[id].tailrepairs++;
-                    search->refcount = 0;
-                    do_item_unlink(search);
-                    break;
-                }
-            }
-            it = slabs_alloc(ntotal, id);
-            if (it == 0) {
-                return NULL;
-            }
+            return NULL;
         }
     }
 
@@ -250,7 +229,6 @@ static void item_unlink_q(item *it) {
 }
 
 int do_item_link(item *it) {
-    MEMCACHED_ITEM_LINK(ITEM_key(it), it->nbytes);
     assert((it->it_flags & (ITEM_LINKED|ITEM_SLABBED)) == 0);
     assert(it->nbytes < (1024 * 1024));  /* 1MB max size */
     it->it_flags |= ITEM_LINKED;
@@ -272,7 +250,6 @@ int do_item_link(item *it) {
 }
 
 void do_item_unlink(item *it) {
-    MEMCACHED_ITEM_UNLINK(ITEM_key(it), it->nbytes);
     if ((it->it_flags & ITEM_LINKED) != 0) {
         it->it_flags &= ~ITEM_LINKED;
         STATS_LOCK();
@@ -286,7 +263,6 @@ void do_item_unlink(item *it) {
 }
 
 void do_item_remove(item *it) {
-    MEMCACHED_ITEM_REMOVE(ITEM_key(it), it->nbytes);
     assert((it->it_flags & ITEM_SLABBED) == 0);
     if (it->refcount != 0) {
         it->refcount--;
@@ -299,7 +275,6 @@ void do_item_remove(item *it) {
 }
 
 void do_item_update(item *it) {
-    MEMCACHED_ITEM_UPDATE(ITEM_key(it), it->nbytes);
     if (it->time < current_time - ITEM_UPDATE_INTERVAL) {
         assert((it->it_flags & ITEM_SLABBED) == 0);
 
@@ -312,8 +287,6 @@ void do_item_update(item *it) {
 }
 
 int do_item_replace(item *it, item *new_it) {
-    MEMCACHED_ITEM_REPLACE(ITEM_key(it), it->nbytes,
-                           ITEM_key(new_it), new_it->nbytes);
     assert((it->it_flags & ITEM_SLABBED) == 0);
 
     do_item_unlink(it);
@@ -355,7 +328,7 @@ char *do_item_cachedump(const unsigned int slabs_clsid, const unsigned int limit
 }
 
 char *do_item_stats(int *bytes) {
-    size_t bufleft = (size_t) LARGEST_ID * 360;
+    size_t bufleft = (size_t) LARGEST_ID * 160;
     char *buffer = malloc(bufleft);
     char *bufcurr = buffer;
     rel_time_t now = current_time;
@@ -372,14 +345,9 @@ char *do_item_stats(int *bytes) {
                 "STAT items:%d:number %u\r\n"
                 "STAT items:%d:age %u\r\n"
                 "STAT items:%d:evicted %u\r\n"
-                "STAT items:%d:evicted_time %u\r\n"
-                "STAT items:%d:outofmemory %u\r\n"
-                "STAT items:%d:tailrepairs %u\r\n",
-                    i, sizes[i], i, now - tails[i]->time,
-                    i, itemstats[i].evicted,
-                    i, itemstats[i].evicted_time,
-                    i, itemstats[i].outofmemory,
-                    i, itemstats[i].tailrepairs);
+                "STAT items:%d:outofmemory %u\r\n",
+                    i, sizes[i], i, now - tails[i]->time, i,
+                    itemstats[i].evicted, i, itemstats[i].outofmemory);
             if (linelen + sizeof("END\r\n") < bufleft) {
                 bufcurr += linelen;
                 bufleft -= linelen;
@@ -446,18 +414,7 @@ bool item_delete_lock_over (item *it) {
 /** wrapper around assoc_find which does the lazy expiration/deletion logic */
 item *do_item_get_notedeleted(const char *key, const size_t nkey, bool *delete_locked) {
     item *it = assoc_find(key, nkey);
-    int was_found = 0;
     if (delete_locked) *delete_locked = false;
-
-    if (settings.verbose > 2) {
-        if (it == NULL) {
-            fprintf(stderr, "> NOT FOUND %s", key);
-        } else {
-            fprintf(stderr, "> FOUND KEY %s", ITEM_key(it));
-            was_found++;
-        }
-    }
-
     if (it != NULL && (it->it_flags & ITEM_DELETED)) {
         /* it's flagged as delete-locked.  let's see if that condition
            is past due, and the 5-second delete_timer just hasn't
@@ -467,41 +424,20 @@ item *do_item_get_notedeleted(const char *key, const size_t nkey, bool *delete_l
             it = NULL;
         }
     }
-
-    if (it == NULL && was_found) {
-        fprintf(stderr, " -nuked by delete lock");
-        was_found--;
-    }
-
     if (it != NULL && settings.oldest_live != 0 && settings.oldest_live <= current_time &&
         it->time <= settings.oldest_live) {
         do_item_unlink(it);           /* MTSAFE - cache_lock held */
         it = NULL;
     }
-
-    if (it == NULL && was_found) {
-        fprintf(stderr, " -nuked by flush");
-        was_found--;
-    }
-
     if (it != NULL && it->exptime != 0 && it->exptime <= current_time) {
         do_item_unlink(it);           /* MTSAFE - cache_lock held */
         it = NULL;
     }
 
-    if (it == NULL && was_found) {
-        fprintf(stderr, " -nuked by expire");
-        was_found--;
-    }
-
     if (it != NULL) {
         it->refcount++;
         DEBUG_REFCNT(it, '+');
     }
-
-    if (settings.verbose > 2)
-        fprintf(stderr, "\n");
-
     return it;
 }
 
