@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* $Id$ */
+/* $Id: memcached.h 772 2008-03-30 08:08:45Z dormando $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -39,6 +39,48 @@
 #define IOV_LIST_HIGHWAT 600
 #define MSG_LIST_HIGHWAT 100
 
+/* Binary protocol stuff */
+#define MIN_BIN_PKT_LENGTH 16
+/* flags:32, expiration:32, cas:64 */
+#define BIN_SET_HDR_LEN 16
+/* incr:64, initial:64, expiration:32 */
+#define BIN_INCR_HDR_LEN 20
+/* flags:32, cas:64 */
+#define GET_RES_HDR_LEN (4+8)
+/* timeout:32 */
+#define BIN_DEL_HDR_LEN 4
+#define BIN_PKT_HDR_WORDS (MIN_BIN_PKT_LENGTH/sizeof(uint32_t))
+
+/* Body is a single 64-bit int */
+#define INCR_RES_LEN 8
+/* len(18446744073709551616) + 2 (or so) */
+#define INCR_MAX_STORAGE_LEN 24
+
+#define BIN_REQ_MAGIC 0x80
+#define BIN_RES_MAGIC 0x81
+
+#define CMD_GET 0
+#define CMD_SET 1
+#define CMD_ADD 2
+#define CMD_REPLACE 3
+#define CMD_DELETE 4
+#define CMD_INCR 5
+#define CMD_DECR 6
+#define CMD_QUIT 7
+#define CMD_FLUSH 8
+#define CMD_GETQ 9
+#define CMD_NOOP 10
+#define CMD_VERSION 11
+
+#define ERR_UNKNOWN_CMD 0x81
+#define ERR_OUT_OF_MEMORY 0x82
+
+#define ERR_NOT_FOUND 0x1
+#define ERR_EXISTS 0x2
+#define ERR_TOO_LARGE 0x3
+#define ERR_INVALID_ARGUMENTS 0x4
+#define ERR_NOT_STORED 0x5
+
 /* Get a consistent bool type */
 #if HAVE_STDBOOL_H
 # include <stdbool.h>
@@ -71,13 +113,10 @@ struct stats {
     uint64_t      set_cmds;
     uint64_t      get_hits;
     uint64_t      get_misses;
-    uint64_t      flush_cmds;
     uint64_t      evictions;
     time_t        started;          /* when the process was started */
     uint64_t      bytes_read;
     uint64_t      bytes_written;
-    unsigned int  accepting_conns;  /* whether we are currently accepting */
-    uint64_t      listen_disabled_num;
 };
 
 #define MAX_VERBOSITY_LEVEL 2
@@ -90,6 +129,7 @@ struct settings {
     char *inter;
     int verbose;
     rel_time_t oldest_live; /* ignore existing items older than this */
+    bool managed;          /* if 1, a tracker manages virtual buckets */
     int evict_to_free;
     char *socketpath;   /* path to unix socket if using local socket */
     int access;  /* access mask (a la chmod) for unix domain socket */
@@ -98,9 +138,6 @@ struct settings {
     int num_threads;        /* number of libevent threads to run */
     char prefix_delimiter;  /* character that marks a key prefix (for stats) */
     int detail_enabled;     /* nonzero if we're collecting detailed stats */
-    int reqs_per_event;     /* Maximum number of io to process on each
-                               io-event. */
-    int backlog;
 };
 
 extern struct stats stats;
@@ -146,8 +183,29 @@ enum conn_states {
     conn_swallow,    /** swallowing unnecessary bytes w/o storing */
     conn_closing,    /** closing this connection */
     conn_mwrite,     /** writing out many items sequentially */
+    conn_bin_init,   /** Reinitializing a binary protocol connection */
+    conn_negotiate,  /** Negotiating a protocol */
 };
 
+enum bin_substates {
+    bin_no_state,
+    bin_reading_set_header,
+    bin_reading_cas_header,
+    bin_read_set_value,
+    bin_reading_get_key,
+    bin_reading_del_header,
+    bin_reading_incr_header,
+};
+
+enum protocol {
+    ascii_prot = 3, /* arbitrary value. */
+    ascii_udp_prot,
+    binary_prot,
+    negotiating_prot, /* Discovering the protocol */
+};
+
+#define IS_UDP(x) (x == ascii_udp_prot)
+
 #define NREAD_ADD 1
 #define NREAD_SET 2
 #define NREAD_REPLACE 3
@@ -158,7 +216,8 @@ enum conn_states {
 typedef struct conn conn;
 struct conn {
     int    sfd;
-    int    state;
+    enum conn_states  state;
+    enum bin_substates substate;
     struct event event;
     short  ev_flags;
     short  which;   /** which events were just triggered */
@@ -213,8 +272,9 @@ struct conn {
     char   **suffixcurr;
     int    suffixleft;
 
+    enum protocol protocol;   /* which protocol this connection speaks */
+
     /* data for UDP clients */
-    bool   udp;       /* is this is a UDP "connection" */
     int    request_id; /* Incoming UDP request ID, if this is a UDP "connection" */
     struct sockaddr request_addr; /* Who sent the most recent request */
     socklen_t request_addr_size;
@@ -222,10 +282,23 @@ struct conn {
     int    hdrsize;   /* number of headers' worth of space is allocated */
 
     int    binary;    /* are we in binary mode */
+    int    bucket;    /* bucket number for the next command, if running as
+                         a managed instance. -1 (_not_ 0) means invalid. */
+    int    gen;       /* generation requested for the bucket */
     bool   noreply;   /* True if the reply should not be sent. */
+    /* Binary protocol stuff */
+    /* This is where the binary header goes */
+    uint32_t bin_header[MIN_BIN_PKT_LENGTH/sizeof(uint32_t)];
+    short cmd;
+    int opaque;
+    int keylen;
     conn   *next;     /* Used for generating a list of conn structures */
 };
 
+
+/* number of virtual buckets for a managed instance */
+#define MAX_BUCKETS 32768
+
 /* current time of day (updated periodically) */
 extern volatile rel_time_t current_time;
 
@@ -233,24 +306,22 @@ extern volatile rel_time_t current_time;
  * Functions
  */
 
-void do_accept_new_conns(const bool do_accept);
 conn *do_conn_from_freelist();
 bool do_conn_add_to_freelist(conn *c);
 char *do_suffix_from_freelist();
 bool do_suffix_add_to_freelist(char *s);
 char *do_defer_delete(item *item, time_t exptime);
 void do_run_deferred_deletes(void);
-char *do_add_delta(conn *c, item *item, const bool incr, const int64_t delta,
-                   char *buf);
+char *do_add_delta(item *item, const bool incr, const int64_t delta, char *buf);
 int do_store_item(item *item, int comm);
-conn *conn_new(const int sfd, const int init_state, const int event_flags, const int read_buffer_size, const bool is_udp, struct event_base *base);
+conn *conn_new(const int sfd, const enum conn_states init_state, const int event_flags, const int read_buffer_size, enum protocol prot, struct event_base *base);
 
 
 #include "stats.h"
 #include "slabs.h"
 #include "assoc.h"
 #include "items.h"
-#include "memcached_dtrace.h"
+
 
 /*
  * In multithreaded mode, we wrap certain functions with lock management and
@@ -269,13 +340,11 @@ conn *conn_new(const int sfd, const int init_state, const int event_flags, const
 
 void thread_init(int nthreads, struct event_base *main_base);
 int  dispatch_event_add(int thread, conn *c);
-void dispatch_conn_new(int sfd, int init_state, int event_flags, int read_buffer_size, int is_udp);
+void dispatch_conn_new(int sfd, enum conn_states init_state, int event_flags, int read_buffer_size, enum protocol prot);
 
 /* Lock wrappers for cache functions that are called from main loop. */
-char *mt_add_delta(conn *c, item *item, const int incr, const int64_t delta,
-                   char *buf);
+char *mt_add_delta(item *item, const int incr, const int64_t delta, char *buf);
 void mt_assoc_move_next_bucket(void);
-void mt_accept_new_conns(const bool do_accept);
 conn *mt_conn_from_freelist(void);
 bool  mt_conn_add_to_freelist(conn *c);
 char *mt_suffix_from_freelist(void);
@@ -303,9 +372,8 @@ void  mt_stats_unlock(void);
 int   mt_store_item(item *item, int comm);
 
 
-# define add_delta(c,x,y,z,a)        mt_add_delta(c,x,y,z,a)
+# define add_delta(x,y,z,a)          mt_add_delta(x,y,z,a)
 # define assoc_move_next_bucket()    mt_assoc_move_next_bucket()
-# define accept_new_conns(x)         mt_accept_new_conns(x)
 # define conn_from_freelist()        mt_conn_from_freelist()
 # define conn_add_to_freelist(x)     mt_conn_add_to_freelist(x)
 # define suffix_from_freelist()      mt_suffix_from_freelist()
@@ -335,9 +403,8 @@ int   mt_store_item(item *item, int comm);
 
 #else /* !USE_THREADS */
 
-# define add_delta(c,x,y,z,a)          do_add_delta(c,x,y,z,a)
+# define add_delta(x,y,z,a)          do_add_delta(x,y,z,a)
 # define assoc_move_next_bucket()    do_assoc_move_next_bucket()
-# define accept_new_conns(x)         do_accept_new_conns(x)
 # define conn_from_freelist()        do_conn_from_freelist()
 # define conn_add_to_freelist(x)     do_conn_add_to_freelist(x)
 # define suffix_from_freelist()      do_suffix_from_freelist()
