@@ -5,7 +5,6 @@
  *  $Id$
  */
 #include "memcached.h"
-#include <assert.h>
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -32,7 +31,7 @@ struct conn_queue_item {
     int     init_state;
     int     event_flags;
     int     read_buffer_size;
-    int     is_udp;
+    int     protocol;
     CQ_ITEM *next;
 };
 
@@ -219,14 +218,6 @@ static void create_worker(void *(*func)(void *), void *arg) {
     }
 }
 
-/*
- * Sets whether or not we accept new connections.
- */
-void mt_accept_new_conns(const bool do_accept) {
-    pthread_mutex_lock(&conn_lock);
-    do_accept_new_conns(do_accept);
-    pthread_mutex_unlock(&conn_lock);
-}
 
 /*
  * Pulls a conn structure from the freelist, if one is available.
@@ -351,9 +342,9 @@ static void thread_libevent_process(int fd, short which, void *arg) {
 
     if (NULL != item) {
         conn *c = conn_new(item->sfd, item->init_state, item->event_flags,
-                           item->read_buffer_size, item->is_udp, me->base);
+                           item->read_buffer_size, item->protocol, me->base);
         if (c == NULL) {
-            if (item->is_udp) {
+            if (IS_UDP(item->protocol)) {
                 fprintf(stderr, "Can't listen for events on UDP socket\n");
                 exit(1);
             } else {
@@ -369,37 +360,28 @@ static void thread_libevent_process(int fd, short which, void *arg) {
 }
 
 /* Which thread we assigned a connection to most recently. */
-static int last_thread = 0;
+static int last_thread = -1;
 
 /*
  * Dispatches a new connection to another thread. This is only ever called
  * from the main thread, either during initialization (for UDP) or because
  * of an incoming connection.
  */
-void dispatch_conn_new(int sfd, int init_state, int event_flags,
-                       int read_buffer_size, int is_udp) {
+void dispatch_conn_new(int sfd, enum conn_states init_state, int event_flags,
+                       int read_buffer_size, enum protocol prot) {
     CQ_ITEM *item = cqi_new();
+    int thread = (last_thread + 1) % settings.num_threads;
 
-    int tid = last_thread % (settings.num_threads - 1);
-
-    /* Skip the dispatch thread (0) */
-    tid++;
-    assert(tid != 0);
-    assert(tid < settings.num_threads);
-    LIBEVENT_THREAD *thread = threads + tid;
-
-    last_thread = tid;
+    last_thread = thread;
 
     item->sfd = sfd;
     item->init_state = init_state;
     item->event_flags = event_flags;
     item->read_buffer_size = read_buffer_size;
-    item->is_udp = is_udp;
+    item->protocol = prot;
 
-    cq_push(&thread->new_conn_queue, item);
-
-    MEMCACHED_CONN_DISPATCH(sfd, thread->thread_id);
-    if (write(thread->notify_send_fd, "", 1) != 1) {
+    cq_push(&threads[thread].new_conn_queue, item);
+    if (write(threads[thread].notify_send_fd, "", 1) != 1) {
         perror("Writing to thread notify pipe");
     }
 }
@@ -513,12 +495,11 @@ char *mt_defer_delete(item *item, time_t exptime) {
 /*
  * Does arithmetic on a numeric item value.
  */
-char *mt_add_delta(conn *c, item *item, int incr, const int64_t delta,
-                   char *buf) {
+char *mt_add_delta(item *item, int incr, const int64_t delta, char *buf) {
     char *ret;
 
     pthread_mutex_lock(&cache_lock);
-    ret = do_add_delta(c, item, incr, delta, buf);
+    ret = do_add_delta(item, incr, delta, buf);
     pthread_mutex_unlock(&cache_lock);
     return ret;
 }
@@ -655,7 +636,7 @@ void thread_init(int nthreads, struct event_base *main_base) {
     pthread_mutex_init(&cqi_freelist_lock, NULL);
     cqi_freelist = NULL;
 
-    threads = calloc(nthreads, sizeof(LIBEVENT_THREAD));
+    threads = malloc(sizeof(LIBEVENT_THREAD) * nthreads);
     if (! threads) {
         perror("Can't allocate thread descriptors");
         exit(1);
